@@ -191,3 +191,24 @@ test('https://domain.com/?', (t) => {
   const serialized = URLSerializer(new URL(domain))
   t.equal(serialized, domain)
 })
+
+// https://github.com/nodejs/undici/issues/2474
+test('hash url', (t) => {
+  t.plan(1)
+  const domain = 'https://domain.com/#a#b'
+  const url = new URL(domain)
+  const serialized = URLSerializer(url, true)
+  t.equal(serialized, url.href.substring(0, url.href.length - url.hash.length))
+})
+
+// https://github.com/nodejs/undici/issues/2474
+test('data url that includes the hash', async (t) => {
+  t.plan(1)
+  const dataURL = 'data:,node#js#'
+  try {
+    const res = await fetch(dataURL)
+    t.equal(await res.text(), 'node')
+  } catch (error) {
+    t.fail(`failed to fetch ${dataURL}`)
+  }
+})
